from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.utils import timezone

from .helpers import Helpers
from .models import Question, TeamChoice, PlayerChoice, CoachChoice
from django.db.models import F
from django.shortcuts import render
from django.urls import reverse
from django.views import generic

class IndexView(generic.ListView):

    seq_size = 17
    fibo_limit = 1500
    nums = range(-100, 100)

    hello_info = ("Let me print you first a fibonacci sequence of size: " +
                  str(seq_size) + " & a max limit (last fibonacci seq # should be < than this #) of: " +
                  str(fibo_limit))
    helper = Helpers()
    fibo_sequence = "Fibonacci Sequence: " + str(helper.get_fibonacci(seq_size, fibo_limit))
    prime_numbers_txt = (" Extra Bonus - Send you the prime numbers until 100 ... Prime Numbers " +
                         "(" + str(nums[0]) + " : " + str(nums[-1]) + "):")
    prime_numbers = str(helper.get_prime_nums(nums))

    template_name = "soccerpoll/index.html"
    context_object_name = "latest_q_list"
    extra_context = {"hello_info": hello_info, "fibo_seq": fibo_sequence,
                     "prime_nums_txt": prime_numbers_txt, "prime_nums": prime_numbers}

    def get_queryset(self):
        """ Return the last 10 published qs (not including future ones)."""
        return Question.objects.filter(pub_date__lte=timezone.now()).order_by('-pub_date')[:10]

#class DetailView(generic.DetailView):
#    model = Question
#    template_name = "spoll/detail.html"
#    def get_queryset(self):
#        """Excludes any questions that aren't published yet."""
#        return Question.objects.filter(pub_date__lte=timezone.now())

class ResultsView(generic.DetailView):
    model = Question
    template_name = "soccerpoll/results.html"

def detail(request, q_id):

    q = get_object_or_404(Question.objects.filter(pub_date__lte=timezone.now()), pk=q_id)

    if q.type.name == "player":
        q_choices = q.playerchoice_set.all
    elif q.type.name == "team":
        q_choices = q.teamchoice_set.all
    else:
        q_choices = q.coachchoice_set.all

    return render(  # Redisplay the question voting form
        request, "soccerpoll/detail.html",
        {"q": q, "q_choices": q_choices
            , "err_msg": "You didn't select a choice ..."}
    )

def vote(request, q_id):
    q = get_object_or_404(Question, pk=q_id)
    selected_choice = get_selected_choice(request, q)
    selected_choice.save()
    # Always return an HttpResponseRedirect after successfully dealing with POST data.
    # This prevents data from being posted twice if a user hits the Back button.
    return HttpResponseRedirect(reverse("soccerpoll:results", args=(q.id,)))

def get_selected_choice(request, q):

    if q.type.name == "player":
        try:
            selected_choice = q.playerchoice_set.get(pk=request.POST["choice"])
        except (KeyError, PlayerChoice.DoesNotExist):
            return get_render(request, q)
        else:
            selected_choice.votes = F("votes") + 1
    elif q.type.name == "team":
        try:
            selected_choice = q.teamchoice_set.get(pk=request.POST["choice"])
        except (KeyError, TeamChoice.DoesNotExist):
            return get_render(request, q)
        else:
            selected_choice.votes = F("votes") + 1
    else:
        try:
            selected_choice = q.coachchoice_set.get(pk=request.POST["choice"])
        except (KeyError, CoachChoice.DoesNotExist):
            return get_render(request, q)
        else:
            selected_choice.votes = F("votes") + 1

    return selected_choice

def get_render(req, q):

    return render(  # Redisplay the question voting form
        req, "soccerpoll/detail.html",
        {"q": q, "error_message": "You didn't select a choice ..."}
    )


import datetime

from django.test import TestCase
from django.utils import timezone
from .models import Question, QuestionType
from django.urls import reverse

class QuestionModelTests(TestCase):

    def test_was_published_recently_with_future_question(self):
        """was_published_recently() returns F 4 qs whose pub_date is in the future."""
        time = timezone.now() + datetime.timedelta(days=30)
        future_q = Question(pub_date=time)
        self.assertIs(future_q.was_published_recently(), False)

    def test_was_published_recently_with_old_question(self):
        """was_published_recently() returns F 4 qs whose pub_date is older than 1 day."""
        time = timezone.now() - datetime.timedelta(days=7, seconds=1)
        old_q = Question(pub_date=time)
        self.assertIs(old_q.was_published_recently(), False)

    def test_was_published_recently_with_recent_question(self):
        """ was_published_recently() returns T 4 qs whose pub_date is within the last day."""
        time = timezone.now() - datetime.timedelta(hours=23, minutes=59, seconds=59)
        recent_q = Question(pub_date=time)
        self.assertIs(recent_q.was_published_recently(), True)

def create_q(text, days, name):
    """ Create a q with the given `text` & published the
    given num of `days` offset to now ("-" for qs published
    in the past, "+" for qs that have yet to be published)."""
    time = timezone.now() + datetime.timedelta(days=days)
    return Question.objects.create(text=text, type=create_q_type(name), pub_date=time)

def create_q_type(name):
    return QuestionType.objects.create(name=name, pub_date=timezone.now())

class QuestionIndexViewTests(TestCase):

    def test_no_qs(self):
        """ If no qs exist, an appropriate msg is displayed."""
        resp = self.client.get(reverse("soccerpoll:index"))
        self.assertEqual(resp.status_code, 200)
        self.assertContains(resp, "No Soccer polls are available ...")
        self.assertQuerySetEqual(resp.context["latest_q_list"], [])

    def test_past_q(self):
        """Qs with a pub_date in the past are shown on the index page."""
        q = create_q(text="Past question.", days=-30, name="player")
        resp = self.client.get(reverse("soccerpoll:index"))
        self.assertEqual(resp.status_code, 200)
        self.assertQuerySetEqual(resp.context["latest_q_list"],[q],)

    def test_future_q(self):
        """Qs with a pub_date in the future aren't show on the index page."""
        create_q(text="Future question.", days=30, name="coach")
        resp = self.client.get(reverse("soccerpoll:index"))
        self.assertEqual(resp.status_code, 200)
        self.assertContains(resp, "No Soccer polls are available ...")
        self.assertQuerySetEqual(resp.context["latest_q_list"], [])

    def test_future_q_and_past_q(self):
        """ Even if both past & future qs exist, only past qs are shown."""
        q1 = create_q(text="Past question.", days=-30, name="player")
        q2 = create_q(text="Future question.", days=30, name="coach")
        print('Publication date - q1: ' + str(q1.pub_date))
        print('Publication date - q2: ' + str(q2.pub_date))
        resp = self.client.get(reverse("soccerpoll:index"))
        self.assertEqual(resp.status_code, 200)
        self.assertQuerySetEqual(resp.context["latest_q_list"],[q1],)

    def test_two_past_qs(self):
        """The qs index page may display multiple qs."""
        q1 = create_q(text="Past question 1.", days=-30, name="team")
        q2 = create_q(text="Past question 2.", days=-5, name="player")
        resp = self.client.get(reverse("soccerpoll:index"))
        self.assertEqual(resp.status_code, 200)
        self.assertQuerySetEqual(resp.context["latest_q_list"],[q2, q1],)
#
class QuestionDetailViewTests(TestCase):

    def test_future_q(self):
        """Detail view of a q with pub_date in the future returns 404."""
        future_q = create_q(text="Future question.", days=5, name="player")
        url = reverse("soccerpoll:detail", args=(future_q.id,))
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)

    def test_past_question(self):
        """Detail view of a q with pub_date in the past shows q text."""
        past_q = create_q(text="Past Question.", days=-5, name="coach")
        url = reverse("soccerpoll:detail", args=(past_q.id,))
        resp = self.client.get(url)
        self.assertContains(resp, past_q.text)



from django.urls import path
from . import views

app_name = 'soccerpoll'
urlpatterns = [
    #This path is mapped to a view template that is mapped to a views.py file -> index method
    path("", views.IndexView.as_view(), name="index"),
    #path("<int:pk>/", views.DetailView.as_view(), name="detail"),
    path("<int:q_id>/", views.detail, name="detail"),
    path("<int:pk>/results/", views.ResultsView.as_view(), name="results"),
    path("<int:q_id>/vote/", views.vote, name="vote")
]


from django.contrib import admin
from .models import (Question, Country, Player, Team, PlayerChoice, TeamChoice, TeamType, QuestionType, Coach, CoachChoice)

class PlayerChoiceInline(admin.TabularInline):
    model = PlayerChoice
    extra = 3

class TeamChoiceInline(admin.TabularInline):
    model = TeamChoice
    extra = 3

class CoachChoiceInline(admin.TabularInline):
    model = CoachChoice
    extra = 3

class QuestionAdmin(admin.ModelAdmin):
    fieldsets = [
        (None, {"fields": ["text", "type"]}),
        ("Creation Date", {"fields": ["pub_date"], "classes": ["collapse"]}),
    ]
    inlines = [PlayerChoiceInline, TeamChoiceInline, CoachChoiceInline]
    list_display = ["text", "type", "pub_date", "was_published_recently"]
    list_filter = ["type", "pub_date"]
    search_fields = ["text", "pub_date"]
    list_per_page = 7
    date_hierarchy = "pub_date"

admin.site.register(QuestionType)
admin.site.register(TeamType)
admin.site.register(Country)
admin.site.register(Player)
admin.site.register(Team)
admin.site.register(Coach)
admin.site.register(Question, QuestionAdmin)
admin.site.register(PlayerChoice)
admin.site.register(TeamChoice)
admin.site.register(CoachChoice)


class Helpers:

    # Algorithm to get prime numbers between +- N & +- N
    def get_prime_nums(self, nums_range):
        primes = []
        nums = []
        for num in nums_range:
            nums.append(num)
        nums.sort()
        for num in nums:
            if num <= 1:
                continue
            cont = 0
            for j in range(1, nums[-1]):
                if num % j == 0:
                    cont = cont + 1
            if cont == 2:
                primes.append(num)
        print('Exist ' + str(len(primes)) + ' prime numbers between: ' + str(nums[0]) + ' & ' + str(nums[-1]) + ':')
        print('Prime numbers: ' + str(primes))
        return primes

    # Algorithm to get fibonacci sequence ... of a specific size & with a defined limit
    def get_fibonacci(self, seq_size, limit):
        x, y, fibo_seq = 0, 1, [0, 1]  # As usual on recursion, to initiate base values
        idx = 1
        while len(fibo_seq) < seq_size and (fibo_seq[-1] + fibo_seq[-2]) < limit:
            x, y = y, x + y
            fibo_seq.append(fibo_seq[idx] + fibo_seq[idx - 1])
            idx += 1
        print('Fibonacci sequence with max seq_size of: ' + str(seq_size) + ' and a limit of: ' + str(limit) + ':')
        print('Fibo Seq: ' + str(fibo_seq))
        return fibo_seq


# Generated by Django 6.0 on 2025-12-08 00:06
import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models

class Migration(migrations.Migration):

    initial = True
    DATE_PUBLISHED='Date Published'
    SPOLL_QUESTION='soccerpoll.question'
    SPOLL_COUNTRY='soccerpoll.country'
    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='Mexico', max_length=200, unique=True)),
                ('abbreviation', models.CharField(default='MX', max_length=200)),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
            ],
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.CharField(max_length=200, verbose_name='Question')),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
            ],
        ),
        migrations.CreateModel(
            name='QuestionType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='player', max_length=50, unique=True)),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
            ],
        ),
        migrations.CreateModel(
            name='TeamType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='league', max_length=50, unique=True)),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
            ],
        ),
        migrations.CreateModel(
            name='Coach',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('age', models.IntegerField(default=0)),
                ('nickname', models.CharField(max_length=100)),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to=SPOLL_COUNTRY)),
            ],
        ),
        migrations.CreateModel(
            name='Player',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('age', models.IntegerField(default=0)),
                ('nickname', models.CharField(max_length=100)),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to=SPOLL_COUNTRY)),
            ],
        ),
        migrations.CreateModel(
            name='PlayerChoice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('votes', models.IntegerField(default=0)),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
                ('choice', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='soccerpoll.player')),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=SPOLL_QUESTION)),
            ],
        ),
        migrations.CreateModel(
            name='CoachChoice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('votes', models.IntegerField(default=0)),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
                ('choice', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='soccerpoll.coach')),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=SPOLL_QUESTION)),
            ],
        ),
        migrations.AddField(
            model_name='question',
            name='type',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.DO_NOTHING, to='soccerpoll.questiontype'),
        ),
        migrations.CreateModel(
            name='Team',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('nickname', models.CharField(max_length=100)),
                ('age', models.IntegerField(default=0)),
                ('uniform_color_local', models.CharField(default='#FFFFFF', max_length=100)),
                ('uniform_color_visitant', models.CharField(default='#000000', max_length=100)),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to=SPOLL_COUNTRY)),
                ('type', models.ForeignKey(default=1, on_delete=django.db.models.deletion.DO_NOTHING, to='soccerpoll.teamtype')),
            ],
        ),
        migrations.CreateModel(
            name='TeamChoice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('votes', models.IntegerField(default=0)),
                ('pub_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name=DATE_PUBLISHED)),
                ('choice', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='soccerpoll.team')),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=SPOLL_QUESTION)),
            ],
        ),
    ]



import datetime

from django.contrib import admin
from django.db import models
from django.utils import timezone

# hierarchy / structure it's django -> db -> models -> Model
# The different model field types (DateTimeField, CharField) correspond to the appropriate HTML input widget.
# Each type of field knows how to display itself in the Django admin.
DATE_PUBLISHED = "Date Published"

class QuestionType(models.Model):

    name = models.CharField(max_length=50, unique=True, default="player")
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return self.name

class Question(models.Model):
    # Django supports all the common database relationships: many-to-one, many-to-many, and one-to-one.
    #Has id & q attributes
    text = models.CharField("Question", max_length=200)
    type = models.ForeignKey(QuestionType, on_delete=models.DO_NOTHING, default=1)
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return str(self.id) + ":" + self.text

    @admin.display(boolean=True, ordering="pub_date", description="Published recently?")
    def was_published_recently(self):
        now = timezone.now()
        return now - datetime.timedelta(days=7) <= self.pub_date <= now

class TeamType(models.Model):

    name = models.CharField(max_length=50, unique=True, default="league")
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return self.name

class Country(models.Model):

    name = models.CharField(max_length=200, unique=True, default="Mexico")
    abbreviation = models.CharField(max_length=200, default="MX")
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return str(self.id) + ":" + self.name

class Player(models.Model):

    name = models.CharField(max_length=200)
    country = models.ForeignKey(Country, on_delete=models.DO_NOTHING)
    age = models.IntegerField(default=0)
    nickname = models.CharField(max_length=100)
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return str(self.id) + ":" + self.name

class Coach(models.Model):

    name = models.CharField(max_length=200)
    country = models.ForeignKey(Country, on_delete=models.DO_NOTHING)
    age = models.IntegerField(default=0)
    nickname = models.CharField(max_length=100)
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return str(self.id) + ":" + self.name

class Team(models.Model):

    name = models.CharField(max_length=200)
    type = models.ForeignKey(TeamType, on_delete=models.DO_NOTHING, default=1)
    country = models.ForeignKey(Country, on_delete=models.DO_NOTHING)
    nickname = models.CharField(max_length=100)
    age = models.IntegerField(default=0)
    uniform_color_local = models.CharField(max_length=100, default="#FFFFFF")
    uniform_color_visitant = models.CharField(max_length=100, default="#000000")
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return str(self.id) + ":" + self.name

class PlayerChoice(models.Model):

    question = models.ForeignKey(Question, on_delete=models.CASCADE)
    choice = models.ForeignKey(Player, on_delete=models.CASCADE)
    votes = models.IntegerField(default=0)
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return str(self.id) + ":" + self.choice.name

class TeamChoice(models.Model):

    question = models.ForeignKey(Question, on_delete=models.CASCADE)
    choice = models.ForeignKey(Team, on_delete=models.CASCADE)
    votes = models.IntegerField(default=0)
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return str(self.id) + ":" + self.choice.name

class CoachChoice(models.Model):

    question = models.ForeignKey(Question, on_delete=models.CASCADE)
    choice = models.ForeignKey(Coach, on_delete=models.CASCADE)
    votes = models.IntegerField(default=0)
    pub_date = models.DateTimeField(DATE_PUBLISHED, default=timezone.now)

    def __str__(self):
        return str(self.id) + ":" + self.choice.name